@@ -1992,6 +1992,10 @@ std::unique_ptr<HistoryView::Media> MediaWebPage::createView(
 		not_null<HistoryView::Element*> message,
 		not_null<HistoryItem*> realParent,
 		HistoryView::Element *replacing) {
+	if (realParent->hideLinks()) {
+		realParent->setHasHiddenLinks(true);
+		return nullptr;
+	}
 	return std::make_unique<HistoryView::WebPage>(message, _page, _flags);
 }
 
@@ -2366,7 +2370,7 @@ TodoListData *MediaTodoList::todolist() const {
 
 TextWithEntities MediaTodoList::notificationText() const {
 	return TextWithEntities()
-		.append(QChar(0x2611))
+		.append(QChar(0x2705))
 		.append(QChar(' '))
 		.append(Ui::Text::Colorized(_todolist->title));
 }
@@ -2594,9 +2598,7 @@ std::unique_ptr<HistoryView::Media> MediaGiftBox::createView(
 			message,
 			HistoryView::GenerateUniqueGiftMedia(message, replacing, unique),
 			HistoryView::MediaGenericDescriptor{
-				.maxWidth = (_data.stargiftReleasedBy
-					? st::msgServiceStarGiftByWidth
-					: st::msgServiceGiftBoxSize.width()),
+				.maxWidth = st::msgServiceGiftBoxSize.width(),
 				.paintBg = HistoryView::UniqueGiftBg(message, unique),
 				.service = true,
 			});
