@@ -66,12 +66,14 @@ class GroupCall;
 class NotifySettings;
 class CustomEmojiManager;
 class Stories;
+class SavedMusic;
 class SavedMessages;
 class Chatbots;
 class BusinessInfo;
 struct ReactionId;
 struct UnavailableReason;
 struct CreditsStatusSlice;
+struct StarsRatingPending;
 struct UniqueGift;
 
 struct RepliesReadTillUpdate {
@@ -183,6 +185,9 @@ public:
 	[[nodiscard]] Stories &stories() const {
 		return *_stories;
 	}
+	[[nodiscard]] SavedMusic &savedMusic() const {
+		return *_savedMusic;
+	}
 	[[nodiscard]] SavedMessages &savedMessages() const {
 		return *_savedMessages;
 	}
@@ -375,6 +380,11 @@ public:
 	void notifyPinnedDialogsOrderUpdated();
 	[[nodiscard]] rpl::producer<> pinnedDialogsOrderUpdated() const;
 
+	void nextForUpgradeGiftInvalidate(not_null<PeerData*> owner);
+	void nextForUpgradeGiftRequest(
+		not_null<PeerData*> owner,
+		Fn<void(std::optional<Data::SavedStarGift>)> done);
+
 	using CreditsSubsRebuilder = rpl::event_stream<CreditsStatusSlice>;
 	using CreditsSubsRebuilderPtr = std::shared_ptr<CreditsSubsRebuilder>;
 	[[nodiscard]] CreditsSubsRebuilderPtr createCreditsSubsRebuilder();
@@ -569,8 +579,8 @@ public:
 		MessageFlags localFlags,
 		NewMessageType type);
 
-	[[nodiscard]] int unreadBadge() const;
-	[[nodiscard]] bool unreadBadgeMuted() const;
+	[[nodiscard]] int unreadWithMentionsBadge() const;
+	[[nodiscard]] bool unreadWithMentionsBadgeMuted() const;
 	[[nodiscard]] int unreadBadgeIgnoreOne(Dialogs::Key key) const;
 	[[nodiscard]] bool unreadBadgeMutedIgnoreOne(Dialogs::Key key) const;
 	[[nodiscard]] int unreadOnlyMutedBadge() const;
@@ -872,11 +882,21 @@ public:
 	[[nodiscard]] int commonStarsPerMessage(
 		not_null<const ChannelData*> channel) const;
 
+	void setPendingStarsRating(StarsRatingPending value);
+	[[nodiscard]] StarsRatingPending pendingStarsRating() const;
+
 	void clearLocalStorage();
 
 private:
 	using Messages = std::unordered_map<MsgId, not_null<HistoryItem*>>;
 
+	struct NextToUpgradeGift {
+		std::optional<Data::SavedStarGift> gift;
+		Fn<void(std::optional<Data::SavedStarGift>)> done;
+		crl::time received = 0;
+		mtpRequestId requestId = 0;
+	};
+
 	void suggestStartExport();
 
 	void setupMigrationViewer();
@@ -1219,6 +1239,7 @@ private:
 	const std::unique_ptr<NotifySettings> _notifySettings;
 	const std::unique_ptr<CustomEmojiManager> _customEmojiManager;
 	const std::unique_ptr<Stories> _stories;
+	const std::unique_ptr<SavedMusic> _savedMusic;
 	const std::unique_ptr<SavedMessages> _savedMessages;
 	const std::unique_ptr<Chatbots> _chatbots;
 	const std::unique_ptr<BusinessInfo> _businessInfo;
@@ -1226,6 +1247,12 @@ private:
 
 	MsgId _nonHistoryEntryId = ShortcutMaxMsgId;
 
+	std::unique_ptr<StarsRatingPending> _pendingStarsRating;
+
+	base::flat_map<
+		not_null<PeerData*>,
+		NextToUpgradeGift> _nextForUpgradeGifts;
+
 	rpl::lifetime _lifetime;
 
 };
