@@ -26,6 +26,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "ui/chat/chat_style.h"
 #include "ui/effects/ripple_animation.h"
 #include "ui/effects/spoiler_mess.h"
+#include "ui/text/custom_emoji_helper.h"
 #include "ui/text/text_options.h"
 #include "ui/text/text_utilities.h"
 #include "ui/painter.h"
@@ -103,21 +104,6 @@ constexpr auto kNonExpandedLinesLimit = 5;
 	return style::colorizeImage(result, white);
 }
 
-[[nodiscard]] TextWithEntities TaskDoneIcon(
-		not_null<Main::Session*> session) {
-	return Ui::Text::SingleCustomEmoji(
-		session->data().customEmojiManager().registerInternalEmoji(
-			MakeTaskDoneImage(),
-			QMargins(0, st::lineWidth, st::lineWidth, 0)));
-}
-
-[[nodiscard]] TextWithEntities TaskIcon(not_null<Main::Session*> session) {
-	return Ui::Text::SingleCustomEmoji(
-		session->data().customEmojiManager().registerInternalEmoji(
-			MakeTaskImage(),
-			QMargins(0, st::lineWidth, st::lineWidth, 0)));
-}
-
 } // namespace
 
 void ValidateBackgroundEmoji(
@@ -320,13 +306,23 @@ void Reply::update(
 		&& !fields.quote.empty();
 	_hasQuoteIcon = hasQuoteIcon ? 1 : 0;
 
-	const auto session = &view->history()->session();
+	const auto repaint = [=] { item->customEmojiRepaint(); };
+	auto helper = Ui::Text::CustomEmojiHelper(Core::TextContext({
+		.session = &view->history()->session(),
+		.repaint = repaint,
+	}));
 	const auto text = (!_displaying && data->unavailable())
 		? TextWithEntities()
 		: task
 		? Ui::Text::Colorized(task->completionDate
-			? TaskDoneIcon(session)
-			: TaskIcon(session)).append(task->text)
+			? helper.image({
+				.image = MakeTaskDoneImage(),
+				.margin = QMargins(0, st::lineWidth, st::lineWidth, 0),
+			})
+			: helper.image({
+				.image = MakeTaskImage(),
+				.margin = QMargins(0, st::lineWidth, st::lineWidth, 0),
+			})).append(task->text)
 		: (message && (fields.quote.empty() || !fields.manualQuote))
 		? message->inReplyText()
 		: !fields.quote.empty()
@@ -343,16 +339,11 @@ void Reply::update(
 			.ignoreTopic = true,
 		}).text
 		: TextWithEntities();
-	const auto repaint = [=] { item->customEmojiRepaint(); };
-	const auto context = Core::TextContext({
-		.session = &view->history()->session(),
-		.repaint = repaint,
-	});
 	_text.setMarkedText(
 		st::defaultTextStyle,
 		text,
 		_multiline ? Ui::ItemTextDefaultOptions() : Ui::DialogTextOptions(),
-		context);
+		helper.context());
 
 	updateName(view, data);
 
@@ -548,15 +539,16 @@ void Reply::updateName(
 		: 0;
 	auto nameFull = TextWithEntities();
 	if (displayAsExternal && !groupNameAdded && !fields.storyId) {
-		nameFull.append(PeerEmoji(history, sender));
+		nameFull.append(PeerEmoji(sender));
 	}
 	nameFull.append(name);
 	if (groupNameAdded) {
-		nameFull.append(' ').append(PeerEmoji(history, externalPeer));
+		nameFull.append(' ').append(PeerEmoji(externalPeer));
 		nameFull.append(externalPeer->name());
 	} else if (originalNameAdded) {
-		nameFull.append(' ').append(ForwardEmoji(&history->owner()));
-		nameFull.append(forwarded->originalSender
+		nameFull.append(' ').append(
+			st::historyReplyForward
+		).append(forwarded->originalSender
 			? forwarded->originalSender->name()
 			: forwarded->originalHiddenSenderInfo->name);
 	}
@@ -934,34 +926,16 @@ void Reply::stopLastRipple() {
 	}
 }
 
-TextWithEntities Reply::PeerEmoji(
-		not_null<History*> history,
-		PeerData *peer) {
-	return PeerEmoji(&history->owner(), peer);
-}
-
-TextWithEntities Reply::PeerEmoji(
-		not_null<Data::Session*> owner,
-		PeerData *peer) {
+TextWithEntities Reply::PeerEmoji(PeerData *peer) {
 	using namespace std;
 	const auto icon = !peer
-		? pair(&st::historyReplyUser, st::historyReplyUserPadding)
+		? &st::historyReplyUser
 		: peer->isBroadcast()
-		? pair(&st::historyReplyChannel, st::historyReplyChannelPadding)
+		? &st::historyReplyChannel
 		: (peer->isChannel() || peer->isChat())
-		? pair(&st::historyReplyGroup, st::historyReplyGroupPadding)
-		: pair(&st::historyReplyUser, st::historyReplyUserPadding);
-	return Ui::Text::SingleCustomEmoji(
-		owner->customEmojiManager().registerInternalEmoji(
-			*icon.first,
-			icon.second));
-}
-
-TextWithEntities Reply::ForwardEmoji(not_null<Data::Session*> owner) {
-	return Ui::Text::SingleCustomEmoji(
-		owner->customEmojiManager().registerInternalEmoji(
-			st::historyReplyForward,
-			st::historyReplyForwardPadding));
+		? &st::historyReplyGroup
+		: &st::historyReplyUser;
+	return Ui::Text::IconEmoji(icon);
 }
 
 TextWithEntities Reply::ComposePreviewName(
@@ -994,11 +968,11 @@ TextWithEntities Reply::ComposePreviewName(
 	auto nameFull = TextWithEntities();
 	using namespace HistoryView;
 	if (displayAsExternal && !groupNameAdded) {
-		nameFull.append(Reply::PeerEmoji(history, sender));
+		nameFull.append(Reply::PeerEmoji(sender));
 	}
 	nameFull.append(shorten ? sender->shortName() : sender->name());
 	if (groupNameAdded) {
-		nameFull.append(' ').append(Reply::PeerEmoji(history, toPeer));
+		nameFull.append(' ').append(Reply::PeerEmoji(toPeer));
 		nameFull.append(toPeer->name());
 	}
 	return (quote
