@@ -446,7 +446,9 @@ QSize Document::countOptimalSize() {
 			auto descriptor = pending
 				? Lottie::IconDescriptor{
 					.name = u"transcribe_loading"_q,
+					.color = &st::historyTextInFg,
 					.sizeOverride = Size(st::historyTranscribeLoadingSize),
+					.colorizeUsingAlpha = true,
 				}
 				: Lottie::IconDescriptor();
 			auto text = (entry.requestId || !entry.shown)
